/*
  Example for Stepper 4 Click

  Date          : August 2017
  Author        : Irena Blagojevic

  Test configuration FT90x :
    
  MCU                : FT900
  Dev. Board         : EasyFT90x v7 
  FT90x Compiler ver : v.2.2.1

  Description :
  Demonstrates how to use Stepper 4 click to move a stepper motor. First
  the speed is set, given in number of steps per revolution. Afterwards
  the motor is moved using STEPPER_step function, passing it the number
  of desired steps. If that number is negative, the motor moves CCW,
  otherwise it moves CW.
*/

#include "MIKROBUS_framework.h"
#include "__STEPPER_Driver.h"

void InitTimerA() {
    TIMER_CONTROL_0 = 2;
    TIMER_SELECT = 0;
    TIMER_PRESC_LS = 0;
    TIMER_PRESC_MS = 0;
    TIMER_WRITE_LS = 15;
    TIMER_WRITE_MS = 39;
    TIMER_CONTROL_3 = 0;
    TIMER_CONTROL_4 |= 17;
    TIMER_CONTROL_2 |= 0;
    TIMER_INT |= 2;
    IRQ_CTRL.B31 = 0;
    TIMER_CONTROL_1 |= 1;
}

void systemInit()
{
    STEPPER_GPIOmap( MIKROBUS1_setAN, MIKROBUS1_setCS, MIKROBUS1_setPWM,
                     MIKROBUS1_setRST, MIKROBUS1_setINT );
    MIKROBUS_gpioDirection( MIKROBUS1, MIKROBUS_AN_PIN, GPIO_OUTPUT );
    MIKROBUS_gpioDirection( MIKROBUS1, MIKROBUS_CS_PIN, GPIO_OUTPUT );
    MIKROBUS_gpioDirection( MIKROBUS1, MIKROBUS_PWM_PIN, GPIO_OUTPUT );
    MIKROBUS_gpioDirection( MIKROBUS1, MIKROBUS_RST_PIN, GPIO_OUTPUT );
    MIKROBUS_gpioDirection( MIKROBUS1, MIKROBUS_INT_PIN, GPIO_OUTPUT );

    InitTimerA();
}

void Stepper_4_Init()
{
    STEPPER_init( 200, 100 );

    STEPPER_setAcc( 5 );
    STEPPER_setDec( 5 );
}

void Stepper_4_Task()
{
    STEPPER_setSpeed( 200 );
    STEPPER_step( 400 );
    Delay_ms( 200 );
    STEPPER_setSpeed( 100 );
    STEPPER_step( -200 );
}

void main()
{
    systemInit();
    Stepper_4_Init();

    Stepper_4_Task();
}


void TimerInterrupt() iv IVT_TIMERS_IRQ{
    if (TIMER_INT_A_bit){
        STEPPER_handler();
        TIMER_INT = (TIMER_INT & 0xAA) | (1 << 0);
    }
}
